/***********************************************************
 * $Id: InsertMailtoLink.java 21 2007-06-07 16:10:43Z lech $
 * 
 * Tapestry Tools
 * 
 * Copyleft (C) 2007 IBCL and EV-i Informationstechnologie GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  
 * Created: 05.06.2007
 *
 * Author: lech
 * 
 ***********************************************************/

package org.clazzes.tapestry.tools.components;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Collection;
import java.util.Vector;

import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.ITemplateComponent;
import org.apache.tapestry.annotations.ComponentClass;
import org.apache.tapestry.annotations.Parameter;
import org.apache.tapestry.engine.IPageLoader;
import org.apache.tapestry.spec.IComponentSpecification;

/**
 * This component allows to easily insert a mail link. Besides patching the
 * mailto: in the link, it allows a little script kiddy protection.
 * 
 * @author lech
 * 
 */
@ComponentClass(allowBody = true, allowInformalParameters = true, reservedParameters = "href")
public abstract class InsertMailtoLink extends BaseComponent implements
        ITemplateComponent {

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tapestry.BaseComponent#renderComponent(org.apache.tapestry.IMarkupWriter,
     *      org.apache.tapestry.IRequestCycle)
     */
    @Override
    protected void renderComponent(IMarkupWriter writer, IRequestCycle cycle) {

        writer.begin("a");
        
        // construct mailto link
        
        boolean replaceAt=(getAtSign()!=null && getAtSign().length()>0 && !getAtSign().equals("@"));
        
        Vector<String> mailtoParams=new Vector<String>();
        
        // to stuff
        String toList=join(getToAdr(),getToAdrs(),", ");
        if (replaceAt) {
            toList=toList.replaceAll("@", getAtSign());
        }
        mailtoParams.add("mailto:"+toList); //.replaceAll(" ", "%20"));
        
        // cc stuff
        String ccList=join(getCcAdr(),getCcAdrs(),", ");
        if (ccList.length()>0) {
            if (replaceAt) {
                ccList=ccList.replaceAll("@", getAtSign());
            }
            mailtoParams.add("cc="+ccList); //.replaceAll(" ", "%20"));
        }
        
        // bcc stuff
        String bccList=join(getBccAdr(),getBccAdrs(),", ");
        if (bccList.length()>0) {
            if (replaceAt) {
                bccList=bccList.replaceAll("@", getAtSign());
            }
            mailtoParams.add("bcc="+bccList); //.replaceAll(" ", "%20"));
        }
        
        // browses and mailclients prefer ISO-8859-1 / Latin-1 here, 
        // but I refuse to play against the W3C on purpose, so UTF-8 is our default
        String urlCharset=getUrlCharset();
        if (urlCharset==null || urlCharset.length()==0)
            urlCharset="UTF-8";
        
        // subject
        String subject=getSubject();
        if (subject!=null && subject.length()>0) {
            try {
                subject=URLEncoder.encode(subject, urlCharset);
            } catch (UnsupportedEncodingException e) { }
            mailtoParams.add("subject="+subject);
        }
        
        // body
        String mailBody=getMailBody();
        if (mailBody!=null && mailBody.length()>0) {
            try {
                mailBody=URLEncoder.encode(mailBody, urlCharset);
            } catch (UnsupportedEncodingException e) { }
            mailtoParams.add("body="+mailBody);
        }
        
        String mailtoRefString=join(null,mailtoParams,"&amp;").replaceFirst("&amp;", "?");
        
        // write href
        writer.attributeRaw("href", mailtoRefString.replaceAll("[ +]","%20"));
        
        // write style, mouseover, ...
        renderInformalParameters(writer, cycle);

        // close opening tag
        writer.closeTag();

        // decide what to write to the body
        boolean writeNameToBody=getToName()!=null && getToName().length()>0;
        boolean writeAdrToBody=!writeNameToBody;
        if (getTagStyle()!=null && getTagStyle().length()>0) {
            if (getTagStyle().equals("toAdr")) {
                writeNameToBody=false;
                writeAdrToBody=true;
            }
            if (getTagStyle().equals("toNameAdr")) {
                writeNameToBody=true;
                writeAdrToBody=true;
            }
            if (getTagStyle().equals("toName")) {
                writeNameToBody=true;
                writeAdrToBody=false;
            }
            if (getTagStyle().equals("none")) {
                writeNameToBody=false;
                writeAdrToBody=false;
            }
        }
        StringBuffer ourBody=new StringBuffer();
        if (writeNameToBody && getToName()!=null && getToName().length()>0) {
            ourBody.append(getToName());
        }
        if (writeAdrToBody && getToAdr()!=null && getToAdr().length()>0) {
            if (writeNameToBody && getToName()!=null && getToName().length()>0) {
                ourBody.append(' ');
                ourBody.append("<"+getToAdr()+">");
            } else {
                ourBody.append(getToAdr());
            }
        }
        if (ourBody.length()>0) {
            String ourBodyString=ourBody.toString();
            if (getAtSign()!=null && getAtSign().length()>0 && !getAtSign().equals("@"))  {
                ourBodyString=ourBody.toString().replaceAll("@", getAtSign());
            }
            if (isTagNbsp()) {
                String[] bodyWords=ourBodyString.split("\\s+");
                boolean first=true;
                for (String bodyWord : bodyWords) {
                    if (first)
                        first=false;
                    else
                        writer.printRaw("&nbsp;");
                    writer.print(bodyWord);
                }
            } else {
                writer.print(ourBodyString);
            }
        }
        
        // rest of body, eventually
        if (isKeepBody())
            renderBody(writer, cycle);

        writer.end();

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tapestry.BaseComponent#finishLoad(org.apache.tapestry.IRequestCycle,
     *      org.apache.tapestry.engine.IPageLoader,
     *      org.apache.tapestry.spec.IComponentSpecification)
     */
    @Override
    public void finishLoad(IRequestCycle cycle, IPageLoader loader,
            IComponentSpecification specification) {
        //super.finishLoad();
    }
    
    /**
     * This attribute is not set to requried in order to allow to use only the
     * toAdrs attribute. Of course at least one toAdr is required for the
     * rendered link to work for the web surfer.
     * 
     * @return The primary recipient.
     */ //(defaultValue = "")
    @Parameter
    public abstract String getToAdr();
    public abstract void setToAdr(String s);

    /**
     * @return The Name of the primary Recipient, only relevant for certain
     *         tagStyle settings.
     */ //(defaultValue = "")
    @Parameter
    public abstract String getToName();

    /**
     * @return The optional wished tag style. Known values are: toAdr,
     *         toNameAdr, toName, none.
     */ //(defaultValue = "toName")
    @Parameter(defaultValue="")
    public abstract String getTagStyle();

    /**
     * @return The optional subject for a mail generated by clicking on the
     *         rendered link.
     */ //(defaultValue = "")
    @Parameter(defaultValue = "")
    public abstract String getSubject();

    /**
     * @return The optional text to print instead of the @. Makes the link
     *         unclickable of course.
     */ //(defaultValue = "@")
    @Parameter(defaultValue = "'@'")
    public abstract String getAtSign();

    /**
     * @return Recipients for the To field.
     */
    @Parameter(defaultValue="null")
    public abstract Collection<String> getToAdrs();

    /**
     * @return One recipients for the Cc field.
     */ //(defaultValue = "")
    @Parameter(defaultValue="")
    public abstract String getCcAdr();

    /**
     * @return Recipients for the Cc field.
     */
    @Parameter
    public abstract Collection<String> getCcAdrs();

    /**
     * @return One recipient for the Bcc field.
     */ //(defaultValue = "")
    @Parameter(defaultValue="")
    public abstract String getBccAdr();

    /**
     * @return Recipients for the Bcc field.
     */
    @Parameter
    public abstract Collection<String> getBccAdrs();

    /**
     * @return The optional mail body template.
     */
    @Parameter(defaultValue = "")
    public abstract String getMailBody();

    /**
     * @return If the body shall be rendered. Set to true if using icons.
     */ 
    @Parameter(defaultValue = "false")
    public abstract boolean isKeepBody();

    /**
     * @return wether to write the tag with &nbsp; for spaces.
     */
    @Parameter(defaultValue = "true")
    public abstract boolean isTagNbsp();
    
    /**
     * @return The charset to url-encode subejct+body. Most clients prefere ISD-8859-1 here, but W3C demands UTF-8
     */
    @Parameter(defaultValue="'UTF-8'")
    public abstract String getUrlCharset();

    /**
     * Joins one string and the elements of a string[], using sep as separator.
     * @param first First string, may be null or empty.
     * @param more Array with more strings, may be null or empty.
     * @param sep Separator. May be null or empty, defaults to ", "
     * @return The joined String.
     */
    protected String join(String first, String[] more, String sep) {
        StringBuffer sb=new StringBuffer();
        // first element
        boolean suppressSep=true;
        if (first!=null && first.length()>0) {
            sb.append(first);
            suppressSep=false;
        }
        // ensure separator
        String mySep=sep;
        if (mySep==null || mySep.length()==0)
            mySep=", ";
        // write more's
        if (more!=null && more.length>0) {
            for (String next : more) {
                if (next==null || next.length()==0)
                    continue;
                if (suppressSep)
                    suppressSep=false;
                else
                    sb.append(mySep);
                sb.append(next);
            }
        }
        return sb.toString();
    }

    /**
     * Joins one string and the elements of a string[], using sep as separator.
     * @param first First string, may be null or empty.
     * @param more Collection with more strings, may be null or empty.
     * @param sep Separator. May be null or empty, defaults to ", "
     * @return The joined String.
     */
    protected String join(String first, Collection<String> more, String sep) {
        StringBuffer sb=new StringBuffer();
        // first element
        boolean suppressSep=true;
        if (first!=null && first.length()>0) {
            sb.append(first);
            suppressSep=false;
        }
        // ensure separator
        String mySep=sep;
        if (mySep==null || mySep.length()==0)
            mySep=", ";
        // write more's
        if (more!=null && more.size()>0) {
            for (String next : more) {
                if (next==null || next.length()==0)
                    continue;
                if (suppressSep)
                    suppressSep=false;
                else
                    sb.append(mySep);
                sb.append(next);
            }
        }
        return sb.toString();
    }

}
