/***********************************************************
 * $Id: LightColorPicker.java 29 2007-06-07 21:02:22Z lech $
 * 
 * Tapestry Tools
 * 
 * Copyleft (C) 2007 IBCL and EV-i Informationstechnologie GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  
 * Author: Wolfgang Glas/ev-i
 * Created: 05.06.2007
 * 
 * Ported to Tapestry 4.1 by Christoph Lechleitner 2007-06-07 
 * 
 ***********************************************************/
 
package org.clazzes.tapestry.tools.components;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tapestry.IAsset;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.IScript;
import org.apache.tapestry.PageRenderSupport;
import org.apache.tapestry.TapestryUtils;
import org.apache.tapestry.annotations.Asset;
import org.apache.tapestry.annotations.ComponentClass;
import org.apache.tapestry.annotations.InjectScript;
import org.apache.tapestry.annotations.Parameter;
import org.apache.tapestry.form.AbstractFormComponent;

/**
 * @author wglas
 * 
 * This component allows to choose a color in HTML notation. The color may also
 * be null.
 */
@ComponentClass(allowInformalParameters=true,allowBody=false)
public abstract class LightColorPicker extends AbstractFormComponent {
    
    protected static Log log=LogFactory.getLog(LightColorPicker.class);
    
    public static final String LINK_ID_PROP = "org.clazzes.tapestry.tools.components.form.LightColorPicker::lastId";

    /**
     * Constructs an instance of LightColorPicker.
     */
    public LightColorPicker() {
        super();
    }

    @InjectScript("form/LightColorPicker.script")
    public abstract IScript getScript();

    @Parameter(required=true)
    public abstract String getValue();
    public abstract void setValue(String s);
    
    @Parameter(required=false)
    public abstract String getLabel();
    //public abstract void setLabel(String l);
    
    @Asset("/org/clazzes/tapestry/tools/components/form/LightColorPicker.css")
    public abstract IAsset getStylesheet();

    //<property-specification name="name" type="java.lang.String"/>
    //<property-specification name="form" type="org.apache.tapestry.IForm"/>

    /**
     * generates an id for the current reincarnation of this link component
     * inside a given page. For the first instance of this component we return
     * 1, for the second 2 and so forth.
     * 
     * @param cycle
     * @return
     */
    private Integer fetchMyId(IRequestCycle cycle) {
        Object idObj = cycle.getAttribute(LINK_ID_PROP);
        Integer id;

        if (idObj != null) {
            id = (Integer) idObj;
            ++id;
        } else
            id = new Integer(1);

        cycle.setAttribute(LINK_ID_PROP, id);

        return id;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.tapestry.AbstractComponent#renderComponent(org.apache.tapestry.IMarkupWriter,
     *      org.apache.tapestry.IRequestCycle)
     */
    @SuppressWarnings("unchecked")
    @Override
    protected void renderFormComponent(IMarkupWriter writer, IRequestCycle cycle) {
        // IForm form = getForm(cycle);

        /*
        log.debug("renderFormComponent() called, "
                +"form.rewinding="+(getForm().isRewinding() ? "true" : "false")
                +"cycle.rewinding="+(cycle.isRewinding() ? "true" : "false")
                +"value="+getValue());
        */
        
        // String name = form.getElementId(this);
        String name = getName();

        // It isn't enough to know whether the cycle in general is rewinding,
        // need to know
        // specifically if the form which contains this component is rewinding.
        if (!getForm().isRewinding() && cycle.isRewinding())
            return; // no reason to render
        
        
        String id = "lcp" + fetchMyId(cycle);
        String col = getValue();
        if (col == null || col.length()==0)
            col = "x";

        if (!isDisabled()) {
            Map symbols = new HashMap();

            symbols.put("id", id);
            symbols.put("col", col);

            PageRenderSupport pageRenderSupport = TapestryUtils
                    .getPageRenderSupport(cycle, this);
            getScript().execute(this, cycle, pageRenderSupport, symbols);
            
            String label=getLabel();
            if (label==null || label.length()==0)
                label="Color "+getName();
            
            // stylesheet
            writer.beginEmpty("link");
            writer.attribute("rel","stylesheet");
            writer.attribute("type","text/css");
            writer.attribute("href",getStylesheet().buildURL());
            writer.closeTag();
            
            // table header start
            writer.begin("table");
            writer.attribute("cellspacing", "2");
            writer.attribute("cellpadding", "2");

            writer.begin("tr");
            writer.begin("td");

            writer.begin("div");
            writer.attribute("id", "colTarget_" + id);
            writer.attribute("onClick", "chooseColor('" + id + "')");
            writer.attribute("class", "colortarget");
            //writer.print("[Choose color "+label+"]");
            writer.end(); // div

            writer.end();// td

            writer.begin("td");
            writer.print(label);
            writer.end();// td

            writer.end();// tr
            writer.end();// table

        }
        getForm().addHiddenValue(name, id, col);
    }

    /**
     * @see org.apache.tapestry.form.AbstractFormComponent#rewindFormComponent(org.apache.tapestry.IMarkupWriter,
     *      org.apache.tapestry.IRequestCycle)
     */
    protected void rewindFormComponent(IMarkupWriter writer, IRequestCycle cycle) {
        /*
        log.debug("rewindFormComponent() called, "
                +"form.rewinding="+(getForm().isRewinding() ? "true" : "false")
                +"cycle.rewinding="+(cycle.isRewinding() ? "true" : "false")
                +"value="+getValue());
        */
        if (!isDisabled()) {
        String col = cycle.getParameter(getName());
        if (col.length() != 7 || col.charAt(0) != '#')
            col = null;
        setValue(col);
        }
    }

}
