/***********************************************************
 * $Id: FormatTools.java 15 2007-06-05 21:56:35Z lech $
 * 
 * Tapestry Tools
 * 
 * Copyright (C) 2007 IBCL and EV-i Informationstechnologie GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *  
 * Created: 05.06.2007
 *
 * Author: lech
 * 
 ***********************************************************/

package org.clazzes.tapestry.tools.util;

import java.util.Calendar;

import org.apache.commons.lang.time.FastDateFormat;

/**
 * @author lech
 *
 */
public class FormatTools {
    
    protected FastDateFormat dateFormatter;
    protected FastDateFormat timeFormatter;
    protected FastDateFormat dateTimeFormatter;
    
    /**
     * Returns a snippet of a text, i.e. the first length chars plus ..., or the whole string if it is no longer than length.
     * @param s Text to snip of.
     * @param length max length of string to return.
     * @return The (beginning of or complete) string s with max. length characters.
     */
    public synchronized final String snipText(String s, int length) {
        if (s==null)
            return "";
        if (s.length() <= length)
            return s;
        return s.substring(0,length-4)+" ...";
    }

    /**
     * Returns a snippet of a text, i.e. the first 128 chars plus ..., or the whole string if it is no longer than 128.
     * @param s Text to snip of.
     * @return The (beginning of or complete) string s with max. 128 characters.
     */
    public synchronized final String snipText(String s) {
        return snipText(s, 128);
    }
    
    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToDate(Calendar calendar) {
        if (calendar==null) 
            return "";
        if (dateFormatter==null) {
          dateFormatter = FastDateFormat.getDateInstance(FastDateFormat.MEDIUM);
        }
        return dateFormatter.format(calendar);
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToTime(Calendar calendar) {
        if (calendar==null) 
            return "";
        if (timeFormatter==null) {
          timeFormatter = FastDateFormat.getTimeInstance(FastDateFormat.SHORT);
        }
        return timeFormatter.format(calendar);
    }

    /**
     * @param calendar
     * @return
     */
    public synchronized final String calendarToDateTime(Calendar calendar) {
        if (calendar==null) 
            return "";
        if (dateTimeFormatter==null) {
          dateTimeFormatter = FastDateFormat.getDateTimeInstance(FastDateFormat.MEDIUM, FastDateFormat.SHORT);
        }
        return dateTimeFormatter.format(calendar);
    }

}
